# -*- coding: utf-8 -*-
from Components.ActionMap import HelpableActionMap
from Components.Sources.List import List

from enigma import gFont, RT_HALIGN_RIGHT, RT_HALIGN_LEFT, RT_VALIGN_CENTER, RT_HALIGN_CENTER

from Screens.HelpMenu import HelpableScreen
from Screens.Screen import Screen
from Screens.MessageBox import MessageBox

from Tools.Directories import fileExists
from Tools.LoadPixmap import LoadPixmap

import json
from time import localtime as tm_localtime, strftime as tm_strftime

from twisted.web.client import getPage, downloadPage
from twisted.internet import defer

from Plugins.Plugin import PluginDescriptor

def main(session, **kwargs):
	if not fileExists("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/apikey.txt"):
		session.open(MessageBox, _("No API-key found. Please provide your API-key in apikey.txt"), MessageBox.TYPE_ERROR, title=_("openweathermap"), timeout=0)
	elif not fileExists("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/location.txt"):
		session.open(MessageBox, _("No location config found. Please provide your location in location.txt in the format latitude,longitude. E.g. 34.799334,6.493301"), MessageBox.TYPE_ERROR, title=_("openweathermap"), timeout=0)
	else:
		session.open(OpenWeatherMap)

ICONBASEURL = "http://openweathermap.org/img/wn/"

def Plugins(path, **kwargs):
	list = [PluginDescriptor(name=_("OpenWeatherMap"),
		description=_("Weather information by OpenWeatherMap"),
		icon="openweathermap.png",
		where=PluginDescriptor.WHERE_PLUGINMENU,
		fnc=main)]
	return list
	
class OpenWeatherMap(Screen, HelpableScreen):
	skin = """
		<screen position="center,60" size="1800,1000" title="OpenWeatherMap">
			<widget source="detailList" render="Listbox" position="20,50" size="400,900" backgroundColor="#00D9D1CC" >
				<convert type="TemplatedMultiContent">
					{
						"templates":
							{
								"default":
									(
										900,
											[
												# date
												MultiContentEntryText(pos=(10,0), size=(380,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=0),
												# weather icon
												MultiContentEntryPixmapAlphaBlend(pos=(110,40), size=(180, 180), png=35, backcolor=0x00D9D1CC),
										
												# temp min
												MultiContentEntryText(pos=(10,200), size=(140,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=10),
												# current temp
												MultiContentEntryText(pos=(10,200), size=(140,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=12),
												# temp label
												MultiContentEntryText(pos=(150,200), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Temp.")),
												# temp max
												MultiContentEntryText(pos=(250,200), size=(140,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=11),
												
												# rain mm
												MultiContentEntryText(pos=(10,240), size=(140,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=28),
												# rain label
												MultiContentEntryText(pos=(150,240), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=31),
												# rain probability
												MultiContentEntryText(pos=(250,240), size=(140,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=27),

												# rain mm
												MultiContentEntryText(pos=(10,240), size=(140,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=28),
												# rain label
												MultiContentEntryText(pos=(150,240), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=29),
												# rain probability
												MultiContentEntryText(pos=(250,240), size=(140,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=27),
												
												# wind km/h
												MultiContentEntryText(pos=(10,280), size=(140,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=21),
												MultiContentEntryText(pos=(150,280), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Wind")),
												# wind direction
												MultiContentEntryText(pos=(250,280), size=(140,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=22),												
												# pressure label
												MultiContentEntryText(pos=(10,340), size=(125,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Pressure")),												
												# humidity label
												MultiContentEntryText(pos=(135,340), size=(130,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Humidity")),	
												# uv index label
												MultiContentEntryText(pos=(265,340), size=(125,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("UV Index")),													
												# pressure
												MultiContentEntryText(pos=(10,380), size=(125,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=18),
												# humidity
												MultiContentEntryText(pos=(135,380), size=(130,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=19),											
												# uv index
												MultiContentEntryText(pos=(265,380), size=(125,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=25),
												# sun / moon label
												MultiContentEntryText(pos=(10,440), size=(380,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Sun / Moon")),
												# sunrise time
												MultiContentEntryText(pos=(10,480), size=(110,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=1),
												# sun rise label
												MultiContentEntryText(pos=(120,480), size=(80,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Rise")),
												# sun set label
												MultiContentEntryText(pos=(200,480), size=(80,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Set")),
												# sunset time
												MultiContentEntryText(pos=(280,480), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=2),
												# moonrise time
												MultiContentEntryText(pos=(10,520), size=(110,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=3),
												# moon rise label
												MultiContentEntryText(pos=(120,520), size=(80,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Rise")),
												# moon set label
												MultiContentEntryText(pos=(200,520), size=(80,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Set")),
												# moonset time
												MultiContentEntryText(pos=(280,520), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=4),
							
												# dew point
												#MultiContentEntryText(pos=(280,580), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, font=0, text=20),									
												
												# label temperature
												MultiContentEntryText(pos=(130,580), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Temperature")),
												# label feels like
												MultiContentEntryText(pos=(260,580), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, color=0x00000000, backcolor=0x00D9D1CC, font=0, text=_("Feels like")),
												# label morning
												MultiContentEntryText(pos=(10,620), size=(120,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x002E86C1, text=_("Morning")),
												# label afternoon
												MultiContentEntryText(pos=(10,660), size=(120,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x00E74C3C, text=_("Afternoon")),
												# label evening
												MultiContentEntryText(pos=(10,700), size=(120,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x00F69C0B, text=_("Evening")),
												# label night
												MultiContentEntryText(pos=(10,740), size=(120,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x000A6CC0, text=_("Night")),
												# temp morning
												MultiContentEntryText(pos=(130,620), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x002E86C1, text=6),
												# temp day
												MultiContentEntryText(pos=(130,660), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x00E74C3C, text=7),
												# temp eve
												MultiContentEntryText(pos=(130,700), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x00F69C0B, text=8),
												# temp night
												MultiContentEntryText(pos=(130,740), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x000A6CC0, text=9),							
												# feels like morning
												MultiContentEntryText(pos=(260,620), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x002E86C1, text=13),
												# feels like day
												MultiContentEntryText(pos=(260,660), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x00E74C3C, text=14),
												# feels like  eve
												MultiContentEntryText(pos=(260,700), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x00F69C0B, text=15),
												# feels like  night
												MultiContentEntryText(pos=(260,740), size=(130,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, backcolor=0x00D9D1CC, font=0, color=0x000A6CC0, text=16),
											]
									),
							},
						"fonts": 
						[
							gFont("Regular", 28),
						],
					}
				</convert>
			</widget>
			<widget source="forecastList" render="Listbox" position="440,50" size="1360, 800" mode="grid" itemWidth="340" itemHeight="380" scrollbarMode="showNever">
				<convert type="TemplatedMultiContent">
					{
						"templates":
							{
								"default":
									(
										380,
											[
												# date
												MultiContentEntryText(pos=(0,0), size=(340,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, font=0, text=0),
												# weather icon
												MultiContentEntryPixmapAlphaBlend(pos=(80,40), size=(180, 180), png=35),
												# temp min
												MultiContentEntryText(pos=(10,200), size=(110,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, font=0, color=0x003498DB, color_sel=0x003498DB, text=10),
												# current temp
												MultiContentEntryText(pos=(10,200), size=(110,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, font=0, text=12),
												# temp label
												MultiContentEntryText(pos=(120,200), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, font=0, text=_("Temp.")),
												# temp max
												MultiContentEntryText(pos=(260,200), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, font=0, color=0x00F20000, color_sel=0x00F20000, text=11),
												# rain label
												MultiContentEntryText(pos=(120,240), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, font=0, text=29),
												# rain label
												MultiContentEntryText(pos=(120,240), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, font=0, text=31),
												# rain mm
												MultiContentEntryText(pos=(10,240), size=(120,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, font=0, text=28),
												# rain probability
												MultiContentEntryText(pos=(260,240), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, font=0, text=27),
												# wind km/h
												MultiContentEntryText(pos=(10,280), size=(110,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, font=0, text=21),
												# wind label
												MultiContentEntryText(pos=(120,280), size=(100,40), flags=RT_HALIGN_CENTER|RT_VALIGN_CENTER, font=0, text=_("Wind")),
												# wind direction
												MultiContentEntryText(pos=(260,280), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, font=0, text=22),
												# uv index
												#MultiContentEntryText(pos=(260,280), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, font=0, text=25),
												# sunrise time
												#MultiContentEntryText(pos=(10,320), size=(110,40), flags=RT_HALIGN_LEFT|RT_VALIGN_CENTER, font=0, text=1),
												# sunset time
												#MultiContentEntryText(pos=(178,320), size=(110,40), flags=RT_HALIGN_RIGHT|RT_VALIGN_CENTER, font=0, text=2),
											]
									),
							},
						"fonts": 
							[
								gFont("Regular", 28),
							],
						"itemHeight": 400,
						"itemWidth": 400,
					}
				</convert>
			</widget>			
		</screen>
		"""
		
	def __init__(self, session):
		self.session = session
		
		Screen.__init__(self, session)
		HelpableScreen.__init__(self)
		
		self["OkCancelActions"] = HelpableActionMap(self, "OkCancelActions",
		{
			"cancel":	self.close,
		}, -1)

		self["DirectionActions"] = HelpableActionMap(self, "DirectionActions",
		{
			"up":			(self.up, _("Move cursor up")),
			"down":			(self.down, _("Move cursor down")),
			"left":			(self.left, _("Page up")),
			"right":		(self.right, _("Page down")),
		}, -1)
		
		self.forecastList = []
		self.detailList = []
		
		self["forecastList"] = List()
		self["detailList"] = List()
		self["detailList"].setBuildFunc(self.buildForecastEntry)

		self.rainPic = None
		self.tempMaxPic = None


		apikey = ""
		latitude = ""
		longitude = ""

		if fileExists("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/apikey.txt"):
			with open("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/apikey.txt", "r") as f:
				apikey = f.read().strip()
				
		if fileExists("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/location.txt"):
			with open("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/location.txt", "r") as f:
				location = f.read().strip()
				if location != "":
					locationData = location.split(",")
					if len(locationData) == 2:
						latitude = locationData[0].strip()
						longitude = locationData[1].strip()
					elif len(locationData) > 2:
						self.session.open(MessageBox, _("You have entered the location in a wrong format. Latitude and longitude must be separated by ',' while position info must be separated by '.'", MessageBox.TYPE_ERROR, title=_("openweathermap"), timeout=0))
					elif len(locationData) < 2:
						self.session.open(MessageBox, _("Location information is incomplete. Please provide it in the format 36.934939,4.399393"), MessageBox.TYPE_ERROR, title=_("openweathermap"), timeout=0)
						
		
		if apikey != "" and longitude != "" and latitude != "":		
			self.callOpenWeatherMapApi( "https://api.openweathermap.org/data/2.5/onecall?lat=%s&lon=%s&exclude=hourly,minutely,alerts&units=metric&appid=%s"  %(latitude, longitude, apikey))

		self.onShown.append(self.setEnabledDisabled)
		self["forecastList"].onSelectionChanged.append(self.updateDetails)
		
	def setEnabledDisabled(self):
		self["forecastList"].setSelectionEnabled(1)
		self["detailList"].setSelectionEnabled(0)
		
	def updateDetails(self):
		cur = self["forecastList"].getCurrent()
		self.detailList = [cur]
		self["detailList"].setList(self.detailList)

	def up(self):
		self["forecastList"].master.content.moveSelection(0)
	
	def down(self):
		self["forecastList"].master.content.moveSelection(1)

	def left(self):
		self["forecastList"].master.content.moveSelection(8)
		
	def right(self):
		self["forecastList"].master.content.moveSelection(9)
		
	def callOpenWeatherMapApi(self, url):
		getPage(url).addCallback(self.getOpenWeatherMapData).addErrback(self.getError)
		
	def getOpenWeatherMapData(self, result):
		self.forecastList = []
		
		ds = []
		weatherDataDict = json.loads(result)
		
		# current weather
		currentWeatherDict = weatherDataDict.get('current', {})
		print "currentWeatherDict", currentWeatherDict
		currentDate = currentWeatherDict.get('dt', None)
		currentSunrise = currentWeatherDict.get('sunrise', None)
		currentSunset = currentWeatherDict.get('sunset', None)
		currentTemp = currentWeatherDict.get('temp', None)
		currentFeelsLike = currentWeatherDict.get('feels_like', None)
		currentPressure = currentWeatherDict.get('pressure', None)
		currentHumidity = currentWeatherDict.get('humidity', None)
		currentDewPoint = currentWeatherDict.get('dew_point', None)
		currentClouds = currentWeatherDict.get('clouds', None)
		currentUvi = currentWeatherDict.get('uvi', None)
		currentVisibility = currentWeatherDict.get('visibility', None)
		currentWindSpeed = currentWeatherDict.get('wind_speed', None)
		currentWindGust = currentWeatherDict.get('wind_gust', None)
		currentWindDegree = currentWeatherDict.get('wind_deg', None)
		currentRainDict = currentWeatherDict.get('rain', {})
		current1hRain = currentRainDict.get('1h', None)
		currentSnowDict = currentWeatherDict.get('snow', {})
		current1hSnow = currentSnowDict.get('1h', None)
		for currentWeatherData in currentWeatherDict.get('weather', []):
			currentWeatherId = currentWeatherData.get('id', None)
			currentWeatherMain = currentWeatherData.get('main', None)
			currentWeatherDescription = currentWeatherData.get('description', None)
			currentWeatherIcon = self.getFormattedString(currentWeatherData.get('icon', ''), "string", "")
			if not fileExists('/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/icons/%s.png' %(currentWeatherIcon)):
				d = downloadPage("%s%s@4x.png" %(ICONBASEURL, currentWeatherIcon), '/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/icons/%s.png' %(currentWeatherIcon))
				d.addCallback(self.downloadFinished, currentWeatherIcon)
				d.addErrback(self.getError, currentWeatherIcon)
				ds.append(d)
			# according to api doc the first entry is primary...let's use this one then
			break
			
		current1h = current1hRain
		current1hType = _("Rain")
		if current1hRain is None:
			current1h = current1hSnow
			current1hType = _("Snow")
		
		
		self.forecastList.append((_("Current Weather"), currentSunrise, currentSunset, None, None, None, None, None, None, None, None, None, currentTemp, None, None, None, None, currentFeelsLike, currentPressure, currentHumidity, currentDewPoint, currentWindSpeed, currentWindDegree, currentWindGust, currentClouds, currentUvi, currentVisibility, None, None, None, current1h, current1hType, currentWeatherId, currentWeatherMain, currentWeatherDescription, currentWeatherIcon))
		
		for forecastDict in weatherDataDict.get('daily', []):
			forecastDate = forecastDict.get('dt', None)
			forecastSunrise = forecastDict.get('sunrise', None)
			forecastSunset = forecastDict.get('sunset', None)
			forecastMoonrise = forecastDict.get('moonrise', None)
			forecastMoonset = forecastDict.get('moonset', None)
			forecastMoonphase = forecastDict.get('moon_phase', None)
			forecastTempDict = forecastDict.get('temp', {})
			forecastDayTemp = forecastTempDict.get('day', None)
			forecastMinTemp = forecastTempDict.get('min', None)
			forecastMaxTemp = forecastTempDict.get('max', None)
			forecastNightTemp = forecastTempDict.get('night', None)
			forecastEveningTemp = forecastTempDict.get('eve', None)
			forecastMorningTemp = forecastTempDict.get('morn', None)
			forecastFeelsDict = forecastDict.get('feels_like', {})
			forecastFeelsDayTemp = forecastFeelsDict.get('day', None)
			forecastFeelsNightTemp = forecastFeelsDict.get('night', None)
			forecastFeelsEveningTemp = forecastFeelsDict.get('eve', None)
			forecastFeelsMorningTemp = forecastFeelsDict.get('morn', None)
			forecastPressure = forecastDict.get('pressure', None)
			forecastHumidity = forecastDict.get('humidity', None)
			forecastDewPoint = forecastDict.get('dew_point', None)
			forecastWindSpeed = forecastDict.get('wind_speed', None)
			forecastWindDegree = forecastDict.get('wind_deg', None)
			forecastWindGust = forecastDict.get('wind_gust', None)
			for forecastWeatherData in forecastDict.get('weather', []):
				forecastWeatherId = forecastWeatherData.get('id', None)
				forecastWeatherMain = forecastWeatherData.get('main', None)
				forecastWeatherDescription = forecastWeatherData.get('description', None)
				forecastWeatherIcon = self.getFormattedString(forecastWeatherData.get('icon'), "string","")
				if not fileExists('/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/icons/%s.png' %(forecastWeatherIcon)):
					d = downloadPage("%s%s@4x.png" %(ICONBASEURL, forecastWeatherIcon), '/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/icons/%s.png' %(forecastWeatherIcon))
					d.addCallback(self.downloadFinished, forecastWeatherIcon)
					d.addErrback(self.getError, forecastWeatherIcon)
					ds.append(d)
				# according to api doc the first entry is primary...let's use this one then
				break

			forecastClouds = forecastDict.get('clouds', None)
			forecastRainPoP = forecastDict.get('pop', None)
			if forecastRainPoP is not None:
				forecastRainPoP = forecastRainPoP*100
			forecastRain = forecastDict.get('rain', 0)

			forecastSnow = forecastDict.get('snow', None)
			
			forecastPercipationType = _("Rain")
			forecastPercipation = forecastRain
			if forecastSnow is not None:
				forecastPercipationType = _("Snow")
				forecastPercipation = forecastSnow
				
			forecastUvi = forecastDict.get('uvi', None)
			
			self.forecastList.append((forecastDate, forecastSunrise, forecastSunset, forecastMoonrise, forecastMoonset, forecastMoonphase, forecastMorningTemp, forecastDayTemp, forecastEveningTemp, forecastNightTemp, forecastMinTemp, forecastMaxTemp, None, forecastFeelsMorningTemp, forecastFeelsDayTemp, forecastFeelsEveningTemp, forecastFeelsNightTemp, None, forecastPressure, forecastHumidity, forecastDewPoint, forecastWindSpeed, forecastWindDegree, forecastWindGust, forecastClouds, forecastUvi, None, forecastRainPoP, forecastPercipation, forecastPercipationType, None, None, forecastWeatherId, forecastWeatherMain, forecastWeatherDescription, forecastWeatherIcon))
			
		if len(ds) > 0:
			dlist = defer.DeferredList(ds, consumeErrors=True)
			dlist.addCallback(self.updateForecastList)
			dlist.addErrback(self.getError, "")
		else:
			print "all icons exist"
			self.updateForecastList(None)
		
	def updateForecastList(self, result):
		print "now update"
		self["forecastList"].setBuildFunc(self.buildForecastEntry)
		self["forecastList"].updateList(self.forecastList)
		self["detailList"].setList([self.forecastList[0]])
	
	def buildForecastEntry(self, date, sunrise, sunset, moonrise, moonset, moonphase, morningTemp, dayTemp, eveningTemp, nightTemp, minTemp, maxTemp, currentTemp, feelsMorningTemp, feelsDayTemp, feelsEveningTemp, feelsNightTemp, currentFeelsLike, pressure, humidity, dewPoint, windSpeed, windDegree, windGust, clouds, uvi, visibility, rainPop, percipation, percipationType, oneHour, oneHourType, weatherId, weatherMain, weatherDescription, weatherIcon):

		if date != _("Current Weather"):
			dateFormatted = self.getFormattedString(date, "date", "")
		else:
			dateFormatted = date
		sunriseFormatted = self.getFormattedString(sunrise, "hours", "")
		sunsetFormatted = self.getFormattedString(sunset, "hours", "")
		moonriseFormatted = self.getFormattedString(moonrise, "hours", "")
		moonsetFormatted = self.getFormattedString(moonset, "hours", "")
		
		morningTempText = self.getFormattedString(morningTemp, "float", "°C")

		dayTempText = self.getFormattedString(dayTemp, "float", "°C")

		eveningTempText = self.getFormattedString(eveningTemp, "float", "°C")

		nightTempText = self.getFormattedString(nightTemp, "float", "°C")

		minTempText = self.getFormattedString(minTemp, "float", "°C")

		maxTempText = self.getFormattedString(maxTemp, "float", "°C")
	
		currentTempText = self.getFormattedString(currentTemp, "float", "°C")

		feelsMorningTempText = self.getFormattedString(feelsMorningTemp, "float", "°C")

		feelsDayTempText = self.getFormattedString(feelsDayTemp, "float", "°C")

		feelsEveningTempText = self.getFormattedString(feelsEveningTemp, "float", "°C")

		feelsNightTempText = self.getFormattedString(feelsNightTemp, "float", "°C")

		currentFeelsLikeText = self.getFormattedString(currentFeelsLike, "float", "°C")

		pressureText = self.getFormattedString(pressure, "int", "hPa")

		humidityText = self.getFormattedString(humidity, "int", "%")

		dewPointText = self.getFormattedString(dewPoint, "float", "°C")

		windSpeedText = self.getFormattedString(windSpeed, "int", "m/s")

		winddirList = ["N", "NO", "O", "SO", "S", "SW", "W", "NW", "N"]
		index = int(round(((windDegree%360)/360.0)*8)%8)
		windDegreeText = winddirList[index]

		windGustText = self.getFormattedString(windGust, "int", "m/s")

		cloudsText = self.getFormattedString(clouds, "int", "%")

		uviText = self.getFormattedString(uvi, "float", "")

		visibilityText = self.getFormattedString(visibility, "int", "m")

		rainPopText = self.getFormattedString(rainPop, "float", "%")

		percipationText = self.getFormattedString(percipation, "float", "mm")

		oneHourText = self.getFormattedString(oneHour, "float", "mm")
		
		weatherPic = None
		if fileExists("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/icons/%s.png" %(weatherIcon)):
			weatherPic = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/openweathermap/icons/%s.png" %(weatherIcon))

		return [
			dateFormatted, 
			sunriseFormatted, 
			sunsetFormatted, 
			moonriseFormatted, 
			moonsetFormatted, 
			moonphase, 
			morningTempText, 
			dayTempText, 
			eveningTempText, 
			nightTempText, 
			minTempText,
			maxTempText, 
			currentTempText, 
			feelsMorningTempText, 
			feelsDayTempText, 
			feelsEveningTempText, 
			feelsNightTempText, 
			currentFeelsLikeText, 
			pressureText, 
			humidityText, 
			dewPointText, 
			windSpeedText, 
			windDegreeText, 
			windGustText, 
			cloudsText, 
			uviText, 
			visibilityText, 
			rainPopText, 
			percipationText, 
			percipationType, 
			oneHourText, 
			oneHourType,
			weatherId, 
			weatherMain, 
			weatherDescription, 
			weatherPic, 
		]	

	def getFormattedString(self, value, valuetype, unit):
		if value is None:
			return ""
		
		if valuetype == "float":
			if unit == "":
				return "%.2f" %(value)
			return "%.2f %s" %(value, unit)
		
		if valuetype == "int":
			if unit == "":
				return "%d" %(value)
			return "%d %s" %(value, unit)
			
		if valuetype == "string":
			newValue = value.encode('utf-8', 'ignore')
			if unit == "":
				return "%s" %(newValue)
			return "%s %s" %(newValue, unit)
			
		if valuetype == "hours":
			return tm_strftime('%H:%M', tm_localtime(value))
			
		if valuetype == "date":
			return tm_strftime('%A, %d.%m.%Y', tm_localtime(value))
		
	def downloadFinished(self, result, icon):
		print "[openweathermap] - finished download of %s" %(icon)
		
	def getError(self, error, icon):
		print "[openweathermap] - error while downloading %s: %s" %(icon, error.getErrorMessage())